#include <algorithm>
#include <array>
#include <bitset>
#include <cassert>
#include <cmath>
#include <cstdint>
#include <iostream>
#include <numeric>
#include <string>
#include <vector>

static constexpr uint32_t N = 3;
static constexpr uint32_t N2 = N * N;
static constexpr uint32_t FULL_BOARD = (1 << N2) - 1;

struct TrapperState {
    uint32_t queryMask{};
    std::array<uint32_t, 2> move{};
    bool canWin = false;
};

bool used[N2 + 1][1 << N2][1 << N2];
TrapperState memo[N2 + 1][1 << N2][1 << N2];

TrapperState canTrapperWin(int32_t r, uint32_t runnerMask, uint32_t unavailable);

// Can the runner win starting from step 4?
std::pair<uint32_t, bool> canRunnerWin(int32_t r, uint32_t runnerMask, uint32_t unavailable) {
    uint32_t newRunnerMask = 0;
    for (uint32_t i = 0; i < N2; ++i) {
        if (!(runnerMask >> i & 1)) continue;

        // left
        if (i % N != 0) {
            newRunnerMask |= (1 << (i - 1));
        }
        // right
        if (i % N != N - 1) {
            newRunnerMask |= (1 << (i + 1));
        }
        // up
        if (i / N != 0) {
            newRunnerMask |= (1 << (i - N));
        }
        // down
        if (i / N != N - 1) {
            newRunnerMask |= (1 << (i + N));
        }
    }

    newRunnerMask &= ~unavailable;

    bool canWin = newRunnerMask ? !canTrapperWin(r - 1, newRunnerMask, unavailable).canWin : false;
    return {newRunnerMask, canWin};
}

// Can the trapper win, starting from step 1?
TrapperState canTrapperWin(const int32_t r, const uint32_t runnerMask, const uint32_t unavailable) {
    if (r == 0) {
        return TrapperState{};
    }

    auto& res = memo[r][runnerMask][unavailable];
    if (used[r][runnerMask][unavailable]) {
        return res;
    }

    for (uint32_t queryMask = 0; queryMask < (1 << N2); ++queryMask) {
        if (queryMask & unavailable) continue;

        std::array<uint32_t, 2> moves{N2, N2};
        for (int32_t j = 0; j < 2; ++j) {
            uint32_t newRunnerMask = j ? runnerMask & queryMask : runnerMask & ~queryMask;
            for (uint32_t i = 0; i < N2; ++i) {
                if (unavailable >> i & 1) continue;
                if (auto [_, canWin] = canRunnerWin(r, newRunnerMask, unavailable | (1 << i)); !canWin) {
                    moves[j] = i;
                    break;
                }
            }
        }

        if (moves[0] != N2 && moves[1] != N2) {
            res = TrapperState{
                .queryMask = queryMask,
                .move = moves,
                .canWin = true,
            };
            break;
        }
    }

    used[r][runnerMask][unavailable] = true;
    return res;
}

void playTrapper(int32_t r, uint32_t runnerMask, uint32_t unavailable) {
    std::string response;
    for (int32_t round = r; round > 0; --round) {
        TrapperState move = canTrapperWin(round, runnerMask, unavailable);
        assert(move.canWin);
        std::cout << std::popcount(move.queryMask) << std::endl;
        if (move.queryMask) {
            for (int32_t j = 0; j < N2; ++j) {
                if (move.queryMask >> j & 1) {
                    std::cout << (j + 1) << ' ';
                }
            }
            std::cout << std::endl;
        }
        std::cin >> response;
        bool hasRunner = response == "Yes";
        std::cout << (move.move[hasRunner] + 1) << std::endl;

        unavailable |= 1 << move.move[hasRunner];

        auto [newRunner, runnerWins] = canRunnerWin(round, runnerMask & (hasRunner ? move.queryMask : ~move.queryMask), unavailable);
        assert(!runnerWins);

        runnerMask = newRunner;

        std::cin >> response;
        if (response == "Trapped") {
            return;
        }
    }
}

bool adj(int32_t x, int32_t y) {
    int32_t r0 = x / N;
    int32_t c0 = x % N;
    int32_t r1 = y / N;
    int32_t c1 = y % N;

    return (r0 == r1 && std::abs(c0 - c1) == 1) || (std::abs(r0 - r1) == 1 && c0 == c1);
}

bool dfs(std::vector<int32_t>& moves, const std::vector<uint32_t>& masks, int32_t idx) {
    if (idx < 0) {
        return true;
    }

    for (int32_t x = 0; x < N2; ++x) {
        if (!(masks[idx] >> x & 1)) continue;
        moves[idx] = x;
        if (idx + 1 < moves.size() && !adj(moves[idx], moves[idx + 1])) continue;
        if (dfs(moves, masks, idx - 1)) {
            return true;
        }
    }

    return false;
}

void playRunner(int32_t r, uint32_t runnerMask, uint32_t unavailable) {
    std::vector<uint32_t> masks;
    masks.reserve(r + 1);
    for (int32_t round = r; round > 0; --round) {
        int32_t n;
        std::cin >> n;

        uint32_t queryMask = 0;
        for (int32_t j = 0; j < n; ++j) {
            uint32_t x;
            std::cin >> x;
            queryMask |= (1 << (x - 1));
        }

        bool won = false;
        for (bool isHit : {false, true}) {
            uint32_t mask = isHit ? runnerMask & queryMask : runnerMask & ~queryMask;
            if (mask == 0) continue;

            bool willWin = true;
            for (uint32_t move = 0; move < N2; ++move) {
                uint32_t newUnavailable = unavailable | (1 << move);
                if (newUnavailable == unavailable) continue;
                willWin &= canRunnerWin(round, mask, newUnavailable).second;
            }

            if (willWin) {
                masks.push_back(mask);
                won = true;
                std::cout << (isHit ? "Yes" : "No") << std::endl;
                int32_t x;
                std::cin >> x;
                unavailable |= (1 << (x - 1));
                auto [newMask, canWin] = canRunnerWin(round, mask, unavailable);
                assert(canWin);
                runnerMask = newMask;
                std::cout << "Free" << std::endl;
                break;
            }
        }
        assert(won);
    }
    masks.push_back(runnerMask);
    std::vector<int32_t> moves(r + 1, N2);
    dfs(moves, masks, r);

    for (int32_t x : moves) {
        std::cout << x + 1 << ' ';
    }
    std::cout << std::endl;
}

int main() {
    int32_t r, u;
    std::cin >> r >> u;
    uint32_t unavailable = 0;
    for (int32_t i = 0; i < u; ++i) {
        int32_t x;
        std::cin >> x;
        --x;

        unavailable |= (1 << x);
    }

    uint32_t available = ~unavailable & FULL_BOARD;
    if (canTrapperWin(r, available, unavailable).canWin) {
        std::cout << "Trapper" << std::endl;
        playTrapper(r, available, unavailable);
    } else {
        std::cout << "Runner" << std::endl;
        playRunner(r, available, unavailable);
    }

    return 0;
}
