// evouga@cs.utexas.edu

#include <iostream>
#include <vector>
#include <unordered_map>
#include <cstdint>

void setbit(uint64_t &mask, int bit)
{
    mask |= (1ULL << bit);
}

void clearbit(uint64_t &mask, int bit)
{
    mask &= ~(1ULL << bit);
}

bool getbit(uint64_t mask, int bit)
{
    return mask & (1ULL << bit);
}

constexpr uint64_t availablemask = 0x1FFULL;
constexpr uint64_t runnermask = 0x3FE00ULL;

struct TrapperMove
{
    int R; // rounds needed to win
    uint64_t query; // where to ask about the Runner
    int yessquare; // where to mark if "yes"
    int nosquare; // where to mark if "no"
};

struct RunnerMove
{
    int R; // rounds until trapped
    bool queryanswer; // true = "yes"
    int yessquare; // worst square to mark if "yes"
    int nosquare; // worst square to mark if "no"
};

void printBoard(uint64_t mask)
{
    for(int y=0; y<3; y++)
    {
        for(int x=0; x<3; x++)
        {
            int idx = 3*y + x;
            if(getbit(mask, idx))
            {
                std::cout << '#';
            }
            else if(getbit(mask, 9 + idx))
            {
                std::cout << '?';
            }
            else
            {
                std::cout << '.';
            }
        }
        std::cout << std::endl;
    }
}

void moveRunner(uint64_t &mask)
{
    uint64_t result = mask & availablemask;
    for(int x=0; x<3; x++)
    {
        for(int y=0; y<3; y++)
        {
            if(getbit(mask, 9 + 3*y + x))
            {
                int dx[] = {1, 0, -1, 0};
                int dy[] = {0, 1, 0, -1};
                for(int k=0; k<4; k++)
                {
                    int newx = x + dx[k];
                    int newy = y + dy[k];
                    if(newx >= 0 && newx < 3 && newy >= 0 && newy < 3)
                    {
                        if(!getbit(mask, 3*newy+newx))
                        {
                            setbit(result, 9 + 3*newy + newx);
                        }
                    }
                }
            }
        }
    }
    mask = result;
}

RunnerMove optimalRunnerMove(uint64_t mask, uint64_t askmask);

std::unordered_map<uint64_t, TrapperMove> memo;

TrapperMove optimalTrapperMove(uint64_t mask)
{
    auto it = memo.find(mask);
    if(it != memo.end())
        return it->second;
        
    std::vector<int> options;
    for(int i=0; i<9; i++)
    {
        if(getbit(mask, 9 + i))
        {
            options.push_back(i);
        }
    }
    
    TrapperMove result;
    result.R = 9;
    
    for(uint64_t submask = 0; submask < (1ULL << options.size()); submask++)
    {
        uint64_t askmask = 0;
        for(int i=0; i<options.size(); i++)
        {
            if(getbit(submask, i))
                setbit(askmask, options[i]);            
        }
        RunnerMove rm = optimalRunnerMove(mask, askmask);
        if(rm.R < result.R)
        {
            result.R = rm.R;
            result.query = askmask;
            result.yessquare = rm.yessquare;
            result.nosquare = rm.nosquare;
        }
    }   
    memo[mask] = result;
    return result;
}

RunnerMove optimalRunnerMove(uint64_t mask, uint64_t askmask)
{
    uint64_t am = mask & availablemask;
    uint64_t rm = mask & runnermask;
    uint64_t yesmask = am | (rm & (askmask << 9));
    uint64_t nomask = am | (rm & ~(askmask << 9));
    
    RunnerMove result;
    
    int bestyes = 9;
    int bestno = 9;
    for(int i=0; i<9; i++)
    {
        if(!getbit(mask, i))
        {
            uint64_t yesoption = yesmask;
            setbit(yesoption, i);
            moveRunner(yesoption);
            if((yesoption & runnermask) == 0)
            {
                bestyes = 1;
                result.yessquare = i;
            }           
            else
            {
                TrapperMove tm = optimalTrapperMove(yesoption);
                if(1 + tm.R < bestyes)
                {
                    bestyes = 1 + tm.R;
                    result.yessquare = i;
                }
            }
            
            uint64_t nooption = nomask;
            setbit(nooption, i);
            moveRunner(nooption);
            
            if((nooption & runnermask) == 0)
            {
                bestno = 1;
                result.nosquare = i;
            }           
            else
            {
                TrapperMove tm = optimalTrapperMove(nooption);
                if(1 + tm.R < bestno)
                {
                    bestno = 1 + tm.R;
                    result.nosquare = i;
                }
            }            
        }
    }
    result.R = std::max(bestyes, bestno);
    result.queryanswer = (bestyes > bestno);    
    return result;    
}

void playAsRunner(int R, uint64_t mask)
{
    std::cout << "Runner" << std::endl;
    
    std::vector<uint64_t> maskhistory;
    
    for(int i=0; i<R; i++)
    {
//        printBoard(mask);
        int N;
        std::cin >> N;
        uint64_t askmask = 0;        
        for(int j=0; j<N; j++)
        {
            int l;
            std::cin >> l;
            setbit(askmask, l-1);
        }
        
        RunnerMove move = optimalRunnerMove(mask, askmask);
        uint64_t am = mask & availablemask;
        uint64_t rm = mask & runnermask;
        if(move.queryanswer)
        {
            std::cout << "Yes" << std::endl;
            mask = am | (rm & (askmask << 9));
        }            
        else
        {
            std::cout << "No" << std::endl;
            mask = am | (rm & ~(askmask << 9));
        }
        maskhistory.push_back(mask);
        
        int unavail;
        std::cin >> unavail;
        setbit(mask, unavail-1);
        
        moveRunner(mask);
        
        std::cout << "Free" << std::endl;        
    }
    maskhistory.push_back(mask);
    
    std::vector<int> squares;
    int curx = 0;
    int cury = 0;
    for(int x=0; x<3; x++)
    {
        for(int y=0; y<3; y++)
        {
            int idx = 3*y + x;
            if(getbit(maskhistory[R], 9 + idx))
            {
                curx = x;
                cury = y;
            }            
        }
    }
    squares.push_back(3*cury + curx);
    for(int i = R-1; i >=0; i--)
    {
        int dx[] = {1, 0, -1, 0};
        int dy[] = {0, 1, 0, -1};
        for(int k=0; k<4; k++)
        {
            int x = curx + dx[k];
            int y = cury + dy[k];
            if(x >= 0 && x < 3 && y >= 0 && y < 3)
            {
                int idx = 3*y + x;
                if(getbit(maskhistory[i], 9 + idx))
                {
                    curx = x;
                    cury = y;
                    break;
                }                
            }
        }
        squares.push_back(3*cury + curx);
    }
    for(int i=R; i>=0; i--)
        std::cout << 1+squares[i] << " ";
    std::cout << std::endl;
}

void playAsTrapper(int R, uint64_t mask)
{
    std::cout << "Trapper" << std::endl;
    
    for(int i=0; i<R; i++)
    {
//        printBoard(mask);
        TrapperMove bestmove = optimalTrapperMove(mask);
        std::vector<int> asks;
        for(int j=0; j<9; j++)
        {
            if(getbit(bestmove.query, j))
            {
                asks.push_back(j+1);
            }
        }
        std::cout << asks.size() << std::endl;
        for(auto it : asks)
            std::cout << it << " ";
        std::cout << std::endl;
        
        uint64_t am = mask & availablemask;
        uint64_t rm = mask & runnermask;
        
        std::string reply;
        std::cin >> reply;
        int marksquare = 0;
        if(reply == "Yes")
        {
            mask = am | (rm & (bestmove.query << 9));
            marksquare = bestmove.yessquare;
        }
        else
        {
            mask = am | (rm & ~(bestmove.query << 9));
            marksquare = bestmove.nosquare;
        }
        
        std::cout << marksquare+1 << std::endl;
        setbit(mask, marksquare);
        
        moveRunner(mask);
        
        std::string status;
        std::cin >> status;
        if(status == "Trapped")
        {
            return;
        }
    }
}

int main()
{
    int R, U;
    std::cin >> R >> U;
    uint64_t mask = 0;
    for(int i=0; i<9; i++)
        setbit(mask, 9 + i);
    if(U > 0)
    {
        for(int i=0; i<U; i++)
        {
            int s;
            std::cin >> s;
            setbit(mask, s-1);
            clearbit(mask, 9 + s-1);
        }
    }

    TrapperMove bestmove = optimalTrapperMove(mask);
    if(bestmove.R > R)
    {
        playAsRunner(R, mask);
    }
    else
    {
        playAsTrapper(R, mask);
    }
}
